/**
 * comparison test SDC vs GT-filtering Only    
 * 
 * @author Michael Stollberg 
 * @version $Revision: 1.2 $ $Date: 2007/10/27 10:07:09 $
 */

package org.deri.wsmx.discovery.caching;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.omwg.ontology.Instance;
import org.omwg.ontology.Ontology;
import org.wsmo.common.IRI;
import org.wsmo.factory.Factory;
import org.wsmo.factory.WsmoFactory;

public class ComparisonSDClightExtendedScenario extends TestCase {

 	protected static Logger logger;
 	private WsmoFactory wsmoFactory; 
 	
	private SDCResourceManager resourceMgr;
	
	private SDCRuntimeDiscoverer sdcDiscoverers;
	private GoalInstanceManager giManager; 


    protected void setUp() throws Exception {
    	super.setUp();
    	
		logger = Logger.getLogger(ComparisonSDClightExtendedScenario.class);
		wsmoFactory = Factory.createWsmoFactory(new HashMap());

		resourceMgr = new SDCResourceManager();
				
		sdcDiscoverers = new SDCRuntimeDiscoverer();
		giManager = new GoalInstanceManager(); 

    }
    
    private List<Integer> runComparisonSingleWSDiscovery(Instance goalInstance, Ontology sdcGraph) {
    	
    	List<Integer> outputList = new ArrayList<Integer>(); 

    	
        // set stop watchers   
        long startTimeSDC;
        long stopTimeSDC;
		    	
    	logger.info("------ running SDC Full RT Discovery ");
    	startTimeSDC = System.currentTimeMillis();
        
    	sdcDiscoverers.discover(goalInstance, sdcGraph); 
    	
    	stopTimeSDC = System.currentTimeMillis();
        long durationSDC = (stopTimeSDC - startTimeSDC); // in ms.        
    	logger.info("------ SDC Full RT Discovery completed in " + durationSDC + " msec \n");
    	outputList.add((int)durationSDC); 
   

        long startTimeGTonly;
        long stopTimeGTonly;

    	logger.info("------ running GT-filtering only Discovery for " + goalInstance.getIdentifier());
    	startTimeGTonly = System.currentTimeMillis();
        
    	sdcDiscoverers.discoverGTonly(goalInstance, sdcGraph); 
    	   	
    	stopTimeGTonly = System.currentTimeMillis();
        long durationGTonly = (stopTimeGTonly - startTimeGTonly); // in ms.        
    	logger.info("------ GT-filtering only Discovery completed in " + durationGTonly + " msec \n");
    	outputList.add((int)durationGTonly);
    	
    	return outputList; 
    
    }

    private List<Integer> runComparisonAllWSDiscovery(Instance goalInstance, Ontology sdcGraph) {
    	
    	logger.info("comparison test run: discovery of all WS ");
    	
    	List<Integer> outputList = new ArrayList<Integer>(); 

    	
        // set stop watchers   
        long startTimeSDC;
        long stopTimeSDC;
		    	
    	logger.info("------ running SDC Full RT Discovery ");
    	startTimeSDC = System.currentTimeMillis();
        
    	sdcDiscoverers.discoverAllWS(goalInstance, sdcGraph); 
    	
    	stopTimeSDC = System.currentTimeMillis();
        long durationSDC = (stopTimeSDC - startTimeSDC); // in ms.        
    	logger.info("------ SDC Full RT Discovery completed in " + durationSDC + " msec \n");
    	outputList.add((int)durationSDC); 


        long startTimeGTonly;
        long stopTimeGTonly;

    	logger.info("------ running GT-filtering only Discovery for " + goalInstance.getIdentifier());
    	startTimeGTonly = System.currentTimeMillis();
        
    	sdcDiscoverers.discoverGTonlyAllWS(goalInstance, sdcGraph); 
    	   	
    	stopTimeGTonly = System.currentTimeMillis();
        long durationGTonly = (stopTimeGTonly - startTimeGTonly); // in ms.        
    	logger.info("------ GT-filtering only Discovery completed in " + durationGTonly + " msec \n");
    	outputList.add((int)durationGTonly); 
    	
    	return outputList; 
    
    }
    
    private void comparisonTestRunner(Instance goalInstance, Ontology sdcGraph, int noOfTestRuns, String theDirectory ) {
    	logger.info("comparison test run: SDC vs GT-only, discovery of Single WS for" +
    			"\n   goal instance: " + goalInstance.getIdentifier() + 
    			"\n   corresponding goal template: " + giManager.getCorrespondingGT(goalInstance, sdcGraph).getIdentifier() 
    			);

    	
    	// getting local name of GI 
    	IRI fullID = (IRI) goalInstance.getIdentifier();
    	String GIlocalName = fullID.getLocalName();
    	
    	String rawDataDirectory = theDirectory + "rawData/"; 
    	
        File dataFile = new File(rawDataDirectory, GIlocalName + "-comparison-data.csv");
        
        BufferedWriter bwData;

        try {
		
		bwData = new BufferedWriter(new FileWriter(dataFile));		
     
        bwData.write("performance comparison for: " + GIlocalName + "\n \n");
        
        bwData.write("RUN, Single WS , SDC (t in msec), GT-Only (t in msec), ALL WS , SDC (t in msec), GT-Only (t in msec) \n");
              	        	           	    
    	for (int run = 1; run < (noOfTestRuns + 1) ; run++) {
    		
    		logger.info("test run no: " + run + "\n" );
    		
    		List<Integer> timesSingleWSdiscovery = runComparisonSingleWSDiscovery(goalInstance, sdcGraph); 
    		
    		List<Integer> timesAllWSdiscovery = runComparisonAllWSDiscovery(goalInstance, sdcGraph); 
    		
    		int timeSDCsingle = timesSingleWSdiscovery.get(0); 
    		int timeGTOnlySingle = timesSingleWSdiscovery.get(1); 

    		int timeSDCall = timesAllWSdiscovery.get(0);  
    		int timeGTOnlyAll = timesAllWSdiscovery.get(1); 
    		
            bwData.write(run + ", , " + timeSDCsingle + "," + timeGTOnlySingle + ", ," + timeSDCall + "," + timeGTOnlyAll + "\n");    		
    		
    	}
       
        bwData.flush();
        bwData.close();

		} catch (IOException e) {
			e.printStackTrace();
		}
    	
    }


    public void testSDCGraphCreator(){
    	
    	logger.info("This runs the runtime discovery comparison test: SDC vs. GT-filtering-only"); 

    	// location for storing the created SDC Graph 
    	String SDCresourceDirectory = "D:/DERI/phd-thesis/repository/stolle/usecases/SWS-challenge/evaluation/runtimeDiscoverySDCvsGTonly/";
    	String sdcGraphFileName = "sdcGraph4extendedSWSC.wsml";
    	String giFile = "goalInstances4extendedSWSC.wsml"; 
		    	

        // loads the SDC Graph 
        Ontology sdcGraph = resourceMgr.loadSDCGraph(SDCresourceDirectory, sdcGraphFileName); 
        // loads the Goal Instances 
    	Ontology goalInstancesKB = resourceMgr.loadSDCGraph(SDCresourceDirectory, giFile);
    	
    	int testRuns = 100; 
    	
    	// tests for individual goal instances
    	
    	
    	Instance goalInstance1 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance1"));
    	
    	comparisonTestRunner(goalInstance1, sdcGraph, testRuns, SDCresourceDirectory); 

    	
    	Instance goalInstance2 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance2"));
    	
    	comparisonTestRunner(goalInstance2, sdcGraph, testRuns, SDCresourceDirectory); 
    	
    	
    	Instance goalInstance3 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance3"));
    	
    	comparisonTestRunner(goalInstance3, sdcGraph, testRuns, SDCresourceDirectory); 
    	
    	
    	Instance goalInstance4 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance4"));
    	
    	comparisonTestRunner(goalInstance4, sdcGraph, testRuns, SDCresourceDirectory); 
    	
    	
    	Instance goalInstance5 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance5"));
    	
    	comparisonTestRunner(goalInstance5, sdcGraph, testRuns, SDCresourceDirectory); 
    	
    	
    	Instance goalInstance6 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance6"));
    	
    	comparisonTestRunner(goalInstance6, sdcGraph, testRuns, SDCresourceDirectory); 
    	
    	
    	Instance goalInstance7 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance7"));
    	
    	comparisonTestRunner(goalInstance7, sdcGraph, testRuns, SDCresourceDirectory); 
    	
    	
    	Instance goalInstance8 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance8"));
    	
    	comparisonTestRunner(goalInstance8, sdcGraph, testRuns, SDCresourceDirectory); 
    	
    	
    	Instance goalInstance9 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance9"));
    	
    	comparisonTestRunner(goalInstance9, sdcGraph, testRuns, SDCresourceDirectory); 
    	
    	
    	Instance goalInstance10 = goalInstancesKB.findInstance(
    			wsmoFactory.createIRI(goalInstancesKB.getDefaultNamespace(), 
    			"goalInstance10"));
    	
    	comparisonTestRunner(goalInstance10, sdcGraph, testRuns, SDCresourceDirectory); 



    }

	
}
