package org.deri.wsmx.discovery.caching;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Set;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.omwg.ontology.Instance;
import org.omwg.ontology.Ontology;
import org.wsmo.common.IRI;
import org.wsmo.common.Namespace;
import org.wsmo.factory.Factory;
import org.wsmo.factory.WsmoFactory;
import org.wsmo.mediator.GGMediator;
import org.wsmo.mediator.WGMediator;
import org.wsmo.service.Goal;
import org.wsmo.service.WebService;

public class MediatorGenerator4SWSC extends TestCase {
	protected static Logger logger = Logger.getLogger(MediatorGenerator4SWSC.class); 
	
	private SDCGraphManager graphMgr; 
	private SDCResourceManager resourceMgr;
	private WsmoFactory wsmoFactory; 
	private SDCGraphCreatorHelper helper; 
	
	protected void setUp() throws Exception {
    	super.setUp();
    	
    	wsmoFactory = Factory.createWsmoFactory(new HashMap());
		
    	graphMgr = new SDCGraphManager(); 
		resourceMgr = new SDCResourceManager();
		helper = new SDCGraphCreatorHelper(); 

    }
	
	public Ontology materializeOmittedWGM(Ontology sdcGraph) {
		
		List<Goal> allRoots = helper.getAllRootNodes(sdcGraph); 
		
		for (Goal aRoot : allRoots) { 
			sdcGraph = stepwiseWGMmaterialization(sdcGraph, aRoot);
						
			while (! (helper.getAllChildNodes(aRoot, sdcGraph).isEmpty()) ) {
				
				List<Goal> childrenOfaRoot = helper.getAllChildNodes(aRoot, sdcGraph); 

				for (Goal aChild : childrenOfaRoot ) {
					sdcGraph = stepwiseWGMmaterialization(sdcGraph, aChild); 
					aRoot = aChild; 
				 }
				
			}
					
		}

		return sdcGraph; 
	}

	public Ontology stepwiseWGMmaterialization(Ontology sdcGraph, Goal theParent) {
		
		List<WebService> usableWS4theGoal = helper.getUsableWS4GT(sdcGraph, theParent, "exact"); 
		List<WebService> usableWS4theGoalPlugin = helper.getUsableWS4GT(sdcGraph, theParent, "plugin");
		
		for (WebService aWS : usableWS4theGoalPlugin ) {
			usableWS4theGoal.add(aWS); 
		}
		
		List<Goal> allDirectChildren = helper.getAllChildNodes(theParent, sdcGraph); 
		
		for (Goal aChild: allDirectChildren ) {
			for (WebService pluginWS : usableWS4theGoal ) {
				sdcGraph = graphMgr.addDiscoveryCacheArc(aChild, pluginWS, sdcGraph, "plugin"); 
				
			}
		}
		
		return sdcGraph; 
	}

	
    public void testMediatorGeneration(){
    	
    	String SDCGraph4SWSCDirectory = "resourcesSWSCshipment/";     	
    	String SDCGraph4SWSCfilename = "sdcGraph4SWSC.wsml" ;
    	
    	Ontology sdcGraph4SWSC = resourceMgr.loadSDCGraph(SDCGraph4SWSCDirectory, SDCGraph4SWSCfilename);
    	
    	
    	String MediatorDirectory = SDCGraph4SWSCDirectory + "WSML/mediators/";
    	
    	Namespace MediatorNamespace = wsmoFactory.createNamespace("swsc", 
    		wsmoFactory.createIRI("http://members.deri.org/~michaels/sdc/swsc-shipment/mediators/"));
    	
    	/*
    	 * generating GG Mediators
    	 */
    	
    	int ggmCounter = 1; 
    	
    	List<Instance> theGoalGraphArcs = graphMgr.getAllGoalGraphArcs(sdcGraph4SWSC);  	
    	
    	for (Instance aGGA : theGoalGraphArcs) {
    		
    		String ggmID = "ggm" + ggmCounter; 
    		IRI ggmIdentifier = wsmoFactory.createIRI(MediatorNamespace, ggmID);

    		GGMediator aGGM = graphMgr.generateGGMediator(sdcGraph4SWSC, aGGA, ggmIdentifier);
        	
    		String ggmFileName = ggmID + ".wsml"; 
        	resourceMgr.storeMediator(MediatorDirectory,ggmFileName,aGGM);
        	
        	ggmCounter += 1; 

    	}

    	/*
    	 * generating WG Mediators (out of explicitly defined discovery cache arcs)
    	 */
    	
    	logger.info("materializing WGMs that are omitted in the SDC Graph .. "); 
    	
    	materializeOmittedWGM(sdcGraph4SWSC); 
    	
    	int wgmCounter = 1; 
    	
    	List<Instance> theDiscoveryCacheArcs = graphMgr.getAllDiscoveryCacheArcs(sdcGraph4SWSC); 	
    	
    	for (Instance aDCA : theDiscoveryCacheArcs) {
    		
    		String wgmID = "wgm" + wgmCounter; 
    		IRI wgmIdentifier = wsmoFactory.createIRI(MediatorNamespace, wgmID);

    		WGMediator aWGM = graphMgr.generateWGMediator(sdcGraph4SWSC, aDCA, wgmIdentifier);
        	
    		String wgmFileName = wgmID + ".wsml"; 
        	resourceMgr.storeMediator(MediatorDirectory,wgmFileName,aWGM);
        	
        	wgmCounter += 1; 

    	}
    	

    	
    	
    }



}
