/**
 * runs the comparison test for the evaluation of the SDC technology  
 * @see SDCNaiveComparison
 * - changeable parameters: goal instance, number of available Web services, number of test runs 
 * - output: result & time for SDC discovery, result & time & number of  matchmaking operations for non-SDC discoverer
 * 
 * @author Michael Stollberg 
 * @version $Revision: 1.2 $ $Date: 2007/10/10 07:16:48 $
 */

package org.deri.wsmx.discovery.caching;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import org.apache.log4j.Logger;
import org.omwg.ontology.Instance;
import org.omwg.ontology.Ontology;
import org.wsmo.common.IRI;
import org.wsmo.factory.Factory;
import org.wsmo.factory.WsmoFactory;

import junit.framework.TestCase;

public class RuntimeDiscoveryAllWSComparisonTester extends TestCase {
	
	protected static Logger logger;
	
	private SDCNaiveComparison discoverers; 
	private SDCResourceManager resourceMgr; 
	
	private WsmoFactory wsmoFactory; 
	
	
    protected void setUp() throws Exception {
    	super.setUp();
    	
    	discoverers = new SDCNaiveComparison();
    	resourceMgr = new SDCResourceManager();
    	
		wsmoFactory = Factory.createWsmoFactory(new HashMap());
		
		logger = Logger.getLogger(RuntimeDiscoveryAllWSComparisonTester.class); 
	
    }
    
    private void comparisonTestRunner(Instance goalInstance, Ontology sdcGraph, Ontology goalInstancesKB, List<Integer> noOfWSList, int noOfTestRuns, String theDirectory ) {
    	logger.info("performance comparison discoverAllWS for: " + goalInstance.getIdentifier());
    	
    	// getting local name of GI 
    	IRI fullID = (IRI) goalInstance.getIdentifier();
    	String GIlocalName = fullID.getLocalName();
    	
        File dataFile = new File(theDirectory, GIlocalName + "discoverAllWS-comparison-data.csv");
        
        BufferedWriter bwData;

        try {
		
		bwData = new BufferedWriter(new FileWriter(dataFile));		
     
        bwData.write("performance comparison discoverAllWS for: " + GIlocalName + "\n \n");
        
        
        for (Integer noOfWS : noOfWSList) {
        	
            bwData.write("no of WS: " + noOfWS + "\n" );
            bwData.write("Test Run, SDC (t in msec), Naive (t in msec), No. Matchmaking Operations (naive) \n");
        	        
    	    	
    	for (int run = 1; run < (noOfTestRuns + 1) ; run++) {
    		
    		logger.info("\n test run no: " + run + "\n" + " -----------SDC Discovery--------------" ); 

    		int timeSDC = discoverers.runSDCDiscovererAllWS(goalInstance, sdcGraph);

    		logger.info("-----------non SDC Discovery--------------" ); 

    		List<Integer> resultNonSDC = discoverers.naiveDiscoveryAllWS(goalInstance, goalInstancesKB, noOfWS);
    		
    		int timeNonSDC = resultNonSDC.get(0); 
    		int noMatchmakingOps= resultNonSDC.get(1); 
    		
            bwData.write(run + "," + timeSDC + "," + timeNonSDC + "," + noMatchmakingOps + "\n");    		
    		
    	}
    	
    	bwData.write("\n"); 
    	
        }        
    	
        bwData.flush();
        bwData.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

    	
    }
    
    public void test() {
    	
    	logger.info("this is the logger for the comparison runs of \n SDC vs. Naive Engine, discovery of all usable Web services for a goal instance");
    	
    	/*
    	 * NOTE 
    	 * The complete comparison test as defined here takes around 8 hours. 
    	 * - for testing, it is strongly recommended to reduce the number of test runs 
    	 * - for statistical analysis, the raw data obtained from the complete test 
    	 *   are available on the SDC homepage 
    	 */
    	
    	
		String SDCgraphDirectory = "resourcesSWSCshipment/";
		String giFile = "goalInstances4SWSC.wsml"; 
		String SDCGraphFile = "sdcGraph4SWSC.wsml";
		
    	String outputDirectory = "outputs/";

    	Ontology goalInstances = resourceMgr.loadSDCGraph(SDCgraphDirectory, giFile); 
    	
    	Ontology sdcGraph = resourceMgr.loadSDCGraph(SDCgraphDirectory, SDCGraphFile);
    	    	    	
    	List<Integer> noOfWSList = new ArrayList<Integer>();
    	
    	noOfWSList.add(10);
    	noOfWSList.add(20);
    	noOfWSList.add(50);
    	noOfWSList.add(100);
    	noOfWSList.add(200);
    	noOfWSList.add(500);
//    	noOfWSList.add(1000);
//    	noOfWSList.add(1500);
//    	noOfWSList.add(2000);
    	
//    	System.out.println(noOfWSList); 
    	
    	int noOfTestRuns = 25; 

    	
    	Instance goalInstance1 = goalInstances.findInstance(
    			wsmoFactory.createIRI(goalInstances.getDefaultNamespace(), 
    			"goalInstance1"));
    	
    	comparisonTestRunner(goalInstance1, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory); 
    	    	
    	
    	Instance goalInstance2 = goalInstances.findInstance(
    			wsmoFactory.createIRI(goalInstances.getDefaultNamespace(), 
    			"goalInstance2"));

    	comparisonTestRunner(goalInstance2, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory);
    	
    	
    	Instance goalInstance3 = goalInstances.findInstance(
    			wsmoFactory.createIRI(goalInstances.getDefaultNamespace(), 
    			"goalInstance3"));

    	comparisonTestRunner(goalInstance3, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory); 


    	Instance goalInstance4 = goalInstances.findInstance(
    			wsmoFactory.createIRI(goalInstances.getDefaultNamespace(), 
    			"goalInstance4"));

    	comparisonTestRunner(goalInstance4, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory); 

    	
    	Instance goalInstance5 = goalInstances.findInstance(
    			wsmoFactory.createIRI(goalInstances.getDefaultNamespace(), 
    			"goalInstance5"));

    	comparisonTestRunner(goalInstance5, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory); 

    	
    	Instance goalInstance6 = goalInstances.findInstance(
    			wsmoFactory.createIRI(goalInstances.getDefaultNamespace(), 
    			"goalInstance6"));

    	comparisonTestRunner(goalInstance6, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory); 

    	
        Instance goalInstance7 = goalInstances.findInstance(
                wsmoFactory.createIRI(goalInstances.getDefaultNamespace(),
                "goalInstance7"));

        comparisonTestRunner(goalInstance7, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory);

        
        
        Instance goalInstance8 = goalInstances.findInstance(
                wsmoFactory.createIRI(goalInstances.getDefaultNamespace(),
                "goalInstance8"));

        comparisonTestRunner(goalInstance8, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory);

        
        
        Instance goalInstance9 = goalInstances.findInstance(
                wsmoFactory.createIRI(goalInstances.getDefaultNamespace(),
                "goalInstance9"));

        comparisonTestRunner(goalInstance9, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory);

    	
        
        Instance goalInstance10 = goalInstances.findInstance(
                wsmoFactory.createIRI(goalInstances.getDefaultNamespace(),
                "goalInstance10"));

        comparisonTestRunner(goalInstance10, sdcGraph, goalInstances, noOfWSList, noOfTestRuns, outputDirectory);

    	
    	
    	
    }


}
