/**
 * performance test for SDC graph creation & management     
 * 
 * @author Michael Stollberg 
 * @version $Revision: 1.1 $ $Date: 2007/10/19 15:56:30 $
 */

package org.deri.wsmx.discovery.caching;

import java.util.Set;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.omwg.ontology.Ontology;
import org.wsmo.common.IRI;
import org.wsmo.common.Namespace;
import org.wsmo.factory.Factory;
import org.wsmo.factory.WsmoFactory;
import org.wsmo.service.Goal;
import org.wsmo.service.WebService;

public class SDCGraphCreationTestOtherOrder extends TestCase {

	private WsmoFactory wsmoFactory; 
	
 	protected static Logger logger;
	private SDCResourceManager theResourceManager; 

	private SDCGraphCreator theCreator; 
	private SDCGraph2DOT dotCreator; 

    protected void setUp() throws Exception {
    	super.setUp();
    	
    	wsmoFactory = Factory.createWsmoFactory(null);

		theResourceManager = new SDCResourceManager();
		theCreator = new SDCGraphCreator();
		dotCreator = new SDCGraph2DOT(); 

		logger = Logger.getLogger(SDCGraphCreator.class);
		
    }
    
    private File createTestWS() {
    	
    	String testWSDirectory = "SDCcreationTestWS";

    	logger.info("trying to create Directory: " + testWSDirectory ); 
    	
    	File wsRepository = new File(testWSDirectory);	
		if (!wsRepository.exists()) {
	    	boolean success = wsRepository.mkdir();
	    	if (success) {
	    		logger.info("Directory: " + wsRepository.getAbsolutePath() + " created");
	    	}    
			
		}
		
    	
    	logger.info("store Web Services in " + wsRepository ); 
    	
    	String wsRepositoryFullPath = wsRepository.getAbsolutePath()+wsRepository.separator; 
    	
		WebService wsMuller = theResourceManager.loadWS("wsMuller.wsml");
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsMuller.wsml", wsMuller); 
		WebService wsRacer = theResourceManager.loadWS("wsRacer.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsRacer.wsml", wsRacer); 
		WebService wsRunner = theResourceManager.loadWS("wsRunner.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsRunner.wsml", wsRunner); 
		WebService wsWalker = theResourceManager.loadWS("wsWalker.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsWalker.wsml", wsWalker); 
		WebService wsWeasel = theResourceManager.loadWS("wsWeasel.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsWeasel.wsml", wsWeasel); 
		WebService wsOther = theResourceManager.loadWS("wsOther.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsOther.wsml", wsOther); 
		


	    for (int i = 2; i < 46 ; i++) {
	    	
			Namespace wsNamespace = wsmoFactory.createNamespace("testWS",
					wsmoFactory.createIRI("http://members.deri.org/~michaels/sdc/swsc-shipment/webservices/")); 
			
			WebService theOriginalWSother = theResourceManager.loadWS("wsOther.wsml");

	    	
	    	String wsID = "wsOther" + i + ".wsml"; 
	    	
			WebService anotherWS = wsmoFactory.createWebService(
					wsmoFactory.createIRI(wsNamespace, wsID));	
			Set<Namespace> theNS = theOriginalWSother.listNamespaces();		
			for (Namespace ns : theNS) {
				anotherWS.addNamespace(ns); 
			}
			Set<Ontology> theOntologies = theOriginalWSother.listOntologies(); 
			for (Ontology ont : theOntologies) {
				anotherWS.addOntology(ont); 
			}
			anotherWS.setCapability(theOriginalWSother.getCapability()); 
			
			theResourceManager.storeWebService(wsRepository.getAbsolutePath()+wsRepository.separator, wsID, anotherWS);
	    } 		
    	
	    return wsRepository; 
    		
    }
    
    private String getLocalGTname(Goal goalTemplate) {

    	IRI fullID = (IRI) goalTemplate.getIdentifier();
    	String GTlocalName = fullID.getLocalName();
    	
    	return GTlocalName; 

    }

    private String getLocalWSname(WebService webservice) {

    	IRI fullID = (IRI) webservice.getIdentifier();
    	String localName = fullID.getLocalName();
    	
    	return localName; 

    }

    private void testRunner(Goal goalTemplate, Ontology sdcGraph, String wsRepository, String theOutputDirectory, BufferedWriter bwData, File csvFile) {

		logger.info( "-----------insertion of goal template: " + getLocalGTname(goalTemplate) +"--------------" ); 
    	
        long startTime;
        long stopTime;
            
        startTime = System.currentTimeMillis();
        stopTime = startTime; 
            
        sdcGraph = theCreator.insertGoalTemplate(sdcGraph, goalTemplate, wsRepository); 	

        stopTime = System.currentTimeMillis();
        long duration = (stopTime - startTime); // in msec        
            
        logger.info("-----------insertion of " + getLocalGTname(goalTemplate) + " completed,  time: " + duration + " msec \n");
		     
        try {        	
            bwData.write("insertion of " + getLocalGTname(goalTemplate) + "," + duration + " \n"); 
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
            

    	
    }

      
    
    public void testSDCGraphCreator(){
    	
    	// the output directory 
    	String outputDirectory = "outputs/"; 
    	// creating the repository of available Web services 
    	File testWSrepositoryFile = createTestWS(); 
    	String testWSrepository = testWSrepositoryFile.getAbsolutePath()+testWSrepositoryFile.separator;    	
    	logger.info("used WS Repository : " + testWSrepository);
   	
	    // loading goal templates 
		Goal gtRoot = theResourceManager.loadGoalTemplate("gtRoot.wsml"); 
    	Goal gt2 = theResourceManager.loadGoalTemplate("gtUS2world.wsml");
    	Goal gt3 = theResourceManager.loadGoalTemplate("gtUS2AF.wsml");	
    	Goal gt4 = theResourceManager.loadGoalTemplate("gtUS2AS.wsml");
    	Goal gt5 = theResourceManager.loadGoalTemplate("gtUS2EU.wsml");
    	Goal gt6 = theResourceManager.loadGoalTemplate("gtUS2EUlight.wsml");
    	Goal gt7 = theResourceManager.loadGoalTemplate("gtUS2NA.wsml");
    	Goal gt8 = theResourceManager.loadGoalTemplate("gtUS2SA.wsml");
    	Goal gt9 = theResourceManager.loadGoalTemplate("gtUS2OC.wsml");
    	Goal gt10 = theResourceManager.loadGoalTemplate("gtNA2NAlight.wsml");
    	
     	
		logger.info( "----------- SDC GRAPH CREATION (subsequent insertion of some goal templates, in a mixed manner) --------------" ); 

    	// set-up CSV file for capturing time 
        File csvFile2 = new File(outputDirectory, "SDCGraphCreationTest-mixed.csv");        
        BufferedWriter bwData2;
        
        try {    		
    		bwData2 = new BufferedWriter(new FileWriter(csvFile2));
    		bwData2.write("Time for SDC Graph Creation (mixed order) \n \n " );   
    		bwData2.write("ACTION, TIME (in msec) \n " );   
    		
    		// subsequent insertion of goal templates (mixed manner) 

    		// loading the SDC Graph Ontology Schema 
    		Ontology sdcGraph1 = theResourceManager.loadSDCGraphSchema();
    		
    		testRunner(gt6, sdcGraph1, testWSrepository, outputDirectory, bwData2, csvFile2);
        	
    		// intermediate storage 
    		String SDCGraphFileName1 = "sdcGraph-partial-1.wsml";
    		theResourceManager.storeSDCGraph(outputDirectory, SDCGraphFileName1, sdcGraph1);
    		// also: creation of DOT file 
    		dotCreator.createDOT(outputDirectory, "sdcGraph-partial-1.wsml", outputDirectory, "sdcGraph-partial-1"); 
    		
    		
    		// loading the SDC Graph Ontology Schema 
    		Ontology sdcGraph2 = theResourceManager.loadSDCGraph(outputDirectory, "sdcGraph-partial-1.wsml"); 
    		
    		testRunner(gt2, sdcGraph2, testWSrepository, outputDirectory, bwData2, csvFile2);
        	
    		// intermediate storage 
    		String SDCGraphFileName2 = "sdcGraph-partial-2.wsml";
    		theResourceManager.storeSDCGraph(outputDirectory, SDCGraphFileName2, sdcGraph2);
    		// also: creation of DOT file 
    		dotCreator.createDOT(outputDirectory, "sdcGraph-partial-2.wsml", outputDirectory, "sdcGraph-partial-2"); 
            

    		// loading the SDC Graph Ontology Schema 
    		Ontology sdcGraph3 = theResourceManager.loadSDCGraph(outputDirectory, "sdcGraph-partial-2.wsml"); 
    		
    		testRunner(gt5, sdcGraph2, testWSrepository, outputDirectory, bwData2, csvFile2);
        	
    		// intermediate storage 
    		String SDCGraphFileName3 = "sdcGraph-partial-3.wsml";
    		theResourceManager.storeSDCGraph(outputDirectory, SDCGraphFileName3, sdcGraph3);
    		// also: creation of DOT file 
    		dotCreator.createDOT(outputDirectory, "sdcGraph-partial-3.wsml", outputDirectory, "sdcGraph-partial-3"); 
            
            // close the CSV file
            bwData2.flush();
            bwData2.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
   		
    }
}
