/**
 * creates the SDC Graph for the SWSC shipment scenario    
 * 
 * @author Michael Stollberg 
 * @version $Revision: 1.1 $ $Date: 2007/10/19 15:56:30 $
 */

package org.deri.wsmx.discovery.caching;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Set;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.omwg.ontology.Ontology;
import org.wsmo.common.IRI;
import org.wsmo.common.Namespace;
import org.wsmo.factory.Factory;
import org.wsmo.factory.WsmoFactory;
import org.wsmo.service.Goal;
import org.wsmo.service.WebService;

public class SDCGraphCreationTestTopDown extends TestCase {

	private WsmoFactory wsmoFactory; 
	
 	protected static Logger logger;
	private SDCResourceManager theResourceManager; 

	private SDCGraphCreator theCreator; 
	private SDCGraphCreatorHelper helper;
	private SDCGraphEvolutionManager evolutionMgr; 


    protected void setUp() throws Exception {
    	super.setUp();
    	
    	wsmoFactory = Factory.createWsmoFactory(null);

		theResourceManager = new SDCResourceManager();
		theCreator = new SDCGraphCreator();
		helper = new SDCGraphCreatorHelper(); 
		evolutionMgr = new SDCGraphEvolutionManager(); 

		logger = Logger.getLogger(SDCGraphCreator.class);
		
    }
    
    private File createTestWS() {
    	
    	String testWSDirectory = "SDCcreationTestWS";

    	logger.info("trying to create Directory: " + testWSDirectory ); 
    	
    	File wsRepository = new File(testWSDirectory);	
		if (!wsRepository.exists()) {
	    	boolean success = wsRepository.mkdir();
	    	if (success) {
	    		logger.info("Directory: " + wsRepository.getAbsolutePath() + " created");
	    	}    
			
		}
		
    	
    	logger.info("store Web Services in " + wsRepository ); 
    	
    	String wsRepositoryFullPath = wsRepository.getAbsolutePath()+wsRepository.separator; 
    	
		WebService wsMuller = theResourceManager.loadWS("wsMuller.wsml");
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsMuller.wsml", wsMuller); 
		WebService wsRacer = theResourceManager.loadWS("wsRacer.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsRacer.wsml", wsRacer); 
		WebService wsRunner = theResourceManager.loadWS("wsRunner.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsRunner.wsml", wsRunner); 
		WebService wsWalker = theResourceManager.loadWS("wsWalker.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsWalker.wsml", wsWalker); 
		WebService wsWeasel = theResourceManager.loadWS("wsWeasel.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsWeasel.wsml", wsWeasel); 
		WebService wsOther = theResourceManager.loadWS("wsOther.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsOther.wsml", wsOther); 
		


	    for (int i = 2; i < 46 ; i++) {
	    	
			Namespace wsNamespace = wsmoFactory.createNamespace("testWS",
					wsmoFactory.createIRI("http://members.deri.org/~michaels/sdc/swsc-shipment/webservices/")); 
			
			WebService theOriginalWSother = theResourceManager.loadWS("wsOther.wsml");

	    	
	    	String wsID = "wsOther" + i + ".wsml"; 
	    	
			WebService anotherWS = wsmoFactory.createWebService(
					wsmoFactory.createIRI(wsNamespace, wsID));	
			Set<Namespace> theNS = theOriginalWSother.listNamespaces();		
			for (Namespace ns : theNS) {
				anotherWS.addNamespace(ns); 
			}
			Set<Ontology> theOntologies = theOriginalWSother.listOntologies(); 
			for (Ontology ont : theOntologies) {
				anotherWS.addOntology(ont); 
			}
			anotherWS.setCapability(theOriginalWSother.getCapability()); 
			
			theResourceManager.storeWebService(wsRepository.getAbsolutePath()+wsRepository.separator, wsID, anotherWS);
	    } 		
    	
	    return wsRepository; 
    		
    }
    
    private String getLocalGTname(Goal goalTemplate) {

    	IRI fullID = (IRI) goalTemplate.getIdentifier();
    	String GTlocalName = fullID.getLocalName();
    	
    	return GTlocalName; 

    }

    private String getLocalWSname(WebService webservice) {

    	IRI fullID = (IRI) webservice.getIdentifier();
    	String localName = fullID.getLocalName();
    	
    	return localName; 

    }

    private void testRunner(Goal goalTemplate, Ontology sdcGraph, String wsRepository, String theOutputDirectory, BufferedWriter bwData, File csvFile) {

		logger.info( "-----------insertion of goal template: " + getLocalGTname(goalTemplate) +"--------------" ); 
    	
        long startTime;
        long stopTime;
            
        startTime = System.currentTimeMillis();
        stopTime = startTime; 
            
        sdcGraph = theCreator.insertGoalTemplate(sdcGraph, goalTemplate, wsRepository); 	

        stopTime = System.currentTimeMillis();
        long duration = (stopTime - startTime); // in msec        
            
        logger.info("-----------insertion of " + getLocalGTname(goalTemplate) + " completed,  time: " + duration + " msec \n");
		     
        try {        	
            bwData.write("insertion of " + getLocalGTname(goalTemplate) + "," + duration + " \n"); 
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
            

    	
    }

      
    
    public void testSDCGraphCreator(){
    	
    	String outputDirectory = "outputs/"; 

    	// creating the repository of available Web services 
    	File testWSrepositoryFile = createTestWS(); 
    	String testWSrepository = testWSrepositoryFile.getAbsolutePath()+testWSrepositoryFile.separator;    	
    	logger.info("used WS Repository : " + testWSrepository);
   	
	    // loading goal templates 
		Goal gtRoot = theResourceManager.loadGoalTemplate("gtRoot.wsml"); 
    	Goal gt2 = theResourceManager.loadGoalTemplate("gtUS2world.wsml");
    	Goal gt3 = theResourceManager.loadGoalTemplate("gtUS2AF.wsml");	
    	Goal gt4 = theResourceManager.loadGoalTemplate("gtUS2AS.wsml");
    	Goal gt5 = theResourceManager.loadGoalTemplate("gtUS2EU.wsml");
    	Goal gt6 = theResourceManager.loadGoalTemplate("gtUS2EUlight.wsml");
    	Goal gt7 = theResourceManager.loadGoalTemplate("gtUS2NA.wsml");
    	Goal gt8 = theResourceManager.loadGoalTemplate("gtUS2SA.wsml");
    	Goal gt9 = theResourceManager.loadGoalTemplate("gtUS2OC.wsml");
    	Goal gt10 = theResourceManager.loadGoalTemplate("gtNA2NAlight.wsml");
    	
     	
 		logger.info( "----------- SDC GRAPH CREATION (subsequent insertion of all goal templates in a top-down manner) --------------" ); 

        // loading the SDC Graph Ontology Schema 
		Ontology theSDCGraph = theResourceManager.loadSDCGraphSchema();

    	// set-up CSV file for capturing time 
        File csvFile = new File(outputDirectory, "SDCGraphCreationTest-topdown.csv");        
        BufferedWriter bwData;
        
        try {    		
    		bwData = new BufferedWriter(new FileWriter(csvFile));
    		bwData.write("Time for SDC Graph Creation (top-down) \n \n " );   
    		bwData.write("ACTION, TIME (in msec) \n " );   
    		
    		// subsequent insertion of goal templates (top-down manner) 
    		testRunner(gtRoot, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);    		
    		testRunner(gt2, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		testRunner(gt3, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		testRunner(gt4, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		testRunner(gt5, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		testRunner(gt6, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		testRunner(gt7, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		testRunner(gt8, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		testRunner(gt9, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		testRunner(gt10, theSDCGraph, testWSrepository, outputDirectory, bwData, csvFile);
    		
    		// close the CSV file 
    		bwData.flush();
            bwData.close();
		} catch (IOException e) {
			e.printStackTrace();
		}

		// store the created SDC graph ontology 
    	String SDCGraphFileName = "sdcGraphOntology-complete.wsml";
		theResourceManager.storeSDCGraph(outputDirectory, SDCGraphFileName, theSDCGraph); 


    }
}
