/**
 * creates the SDC Graph for the SWSC shipment scenario    
 * 
 * @author Michael Stollberg 
 * @version $Revision: 1.3 $ $Date: 2007/10/11 19:42:53 $
 */

package org.deri.wsmx.discovery.caching;

import java.util.ArrayList;
import java.util.List;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.omwg.ontology.Ontology;
import org.wsmo.service.Goal;
import org.deri.wsmx.discovery.caching.matchmaking.*;

public class SDCGraphGenerationExtendedSWSCscenario extends TestCase {

 	protected static Logger logger;
	private SDCResourceManager resourceMgr;
	
	private SDCGraphCreator creator; 


    protected void setUp() throws Exception {
    	super.setUp();

		resourceMgr = new SDCResourceManager();
		
		creator = new SDCGraphCreator(); 
		
		logger = Logger.getLogger(SDCGraphGenerationExtendedSWSCscenario.class);


    }
    
    private void insertSingleGT(Goal goalTemplate, String sdcGraphFileName) {

    	String SDCGraphDirectory = "outputs/";
    	
		// local directory with additional WS descriptions 
    	String theExtendedWSdirectory = "resourcesSWSCshipment/extended/WSML/webservices/";


		// re-load SDC Graph 
		Ontology sdcGraph = resourceMgr.loadSDCGraph(SDCGraphDirectory, sdcGraphFileName); 
		// insert new GT  
		sdcGraph = creator.insertGoalTemplate(sdcGraph, goalTemplate, theExtendedWSdirectory);
		// store updated SDC Graph 
		resourceMgr.storeSDCGraph(SDCGraphDirectory, sdcGraphFileName, sdcGraph); 
    	
    }
    
    public void testSDCGraphCreator(){
    	
    	logger.info("This creates the complete SDC Graph for the Extended SWSC Shipment Scenario"); 

    	// location for storing the created SDC Graph 
    	String SDCGraphDirectory = "outputs/";
    	String sdcGraphFileName = "sdcGraph4extendedSWSC.wsml";
    	

        // loads the SDC Graph Schema 
		Ontology theSDCGraph = resourceMgr.loadSDCGraphSchema();
		
    	// store the SDC graph ontology schema 
    	resourceMgr.storeSDCGraph(SDCGraphDirectory, sdcGraphFileName, theSDCGraph); 

		
		
    	// loading goal templates 
		Goal gtRoot = resourceMgr.loadGoalTemplate("gtRoot.wsml"); 
    	Goal gt2 = resourceMgr.loadGoalTemplate("gtUS2world.wsml");
    	Goal gt3 = resourceMgr.loadGoalTemplate("gtUS2AF.wsml");	
    	Goal gt4 = resourceMgr.loadGoalTemplate("gtUS2AFlight.wsml");
    	Goal gt5 = resourceMgr.loadGoalTemplate("gtUS2AS.wsml");
    	Goal gt6 = resourceMgr.loadGoalTemplate("gtUS2ASlight.wsml");
    	Goal gt7 = resourceMgr.loadGoalTemplate("gtUS2EU.wsml");
    	Goal gt8 = resourceMgr.loadGoalTemplate("gtUS2EUlight.wsml");
    	Goal gt9 = resourceMgr.loadGoalTemplate("gtUS2NA.wsml");
    	Goal gt10 = resourceMgr.loadGoalTemplate("gtUS2SA.wsml");
    	Goal gt11 = resourceMgr.loadGoalTemplate("gtUS2SAlight.wsml");
    	Goal gt12 = resourceMgr.loadGoalTemplate("gtUS2OC.wsml");
    	Goal gt13 = resourceMgr.loadGoalTemplate("gtUS2OClight.wsml");
    	Goal gt14 = resourceMgr.loadGoalTemplate("gtNA2NAlight.wsml");
    	
    	    	    	
        // to stop the time for SDC Graph Creation  
        long startTime;
        long stopTime;
        startTime = System.currentTimeMillis();
        stopTime = startTime;
    	
        insertSingleGT(gtRoot, sdcGraphFileName);  
        insertSingleGT(gt2, sdcGraphFileName);  
        insertSingleGT(gt3, sdcGraphFileName);  
        insertSingleGT(gt4, sdcGraphFileName);  
        insertSingleGT(gt5, sdcGraphFileName);  
        insertSingleGT(gt6, sdcGraphFileName);  
        insertSingleGT(gt7, sdcGraphFileName);  
        insertSingleGT(gt8, sdcGraphFileName);  
        insertSingleGT(gt9, sdcGraphFileName);  
        insertSingleGT(gt10, sdcGraphFileName);  
        insertSingleGT(gt11, sdcGraphFileName);  
        insertSingleGT(gt12, sdcGraphFileName);  
        insertSingleGT(gt13, sdcGraphFileName);  
        insertSingleGT(gt14, sdcGraphFileName);  
        	 	

/* all insertions in 1 step         
		// local directory with additional WS descriptions 
    	String theExtendedWSdirectory = "D:/DERI/phd-thesis/repository/stolle/usecases/SWS-challenge/evaluation/runtimeDiscoverySDCvsGTonly/SWSC-shipmentscenario-extended-modelling/WSML/webservices/";

        // iterative SDC Graph Creation 
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gtRoot, theExtendedWSdirectory); 	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt2, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt3, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt4, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt5, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt6, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt7, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt8, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt9, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt10, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt11, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt12, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt13, theExtendedWSdirectory);   	
    	theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt14, theExtendedWSdirectory);
*/     	   	

/* for extended SWSC shipment scenario without additional WS 

		// local directory with only original WS descriptions 
    	String wsRepository = "D:/DERI/phd-thesis/repository/stolle/usecases/SWS-challenge/own-modelling/WSML/webservices/";

    	// iterative SDC Graph Creation 
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gtRoot, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt2, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt3, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt4, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt5, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt6, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt7, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt8, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt9, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt10, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt11, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt12, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt13, wsRepository);
        theSDCGraph = creator.insertGoalTemplate(theSDCGraph, gt14, wsRepository);

*/        
        // getting the duration 
    	stopTime = System.currentTimeMillis();
        long duration = (stopTime - startTime); // in ms.       
        logger.info("---------- \n TIME NEEDED FOR SDC GRAPH CREATION: "+ duration +" ms \n ----------" ); 

    	// storing created SDC Graph  
//    	resourceMgr.storeSDCGraph(SDCGraphDirectory, sdcGraphFileName, theSDCGraph); 



    }

	
}
