/**
 * performance test for SDC graph maintenance
 * 
 * @author Michael Stollberg 
 * @version $Revision: 1.1 $ $Date: 2007/10/19 15:56:30 $
 */

package org.deri.wsmx.discovery.caching;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Set;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.omwg.ontology.Ontology;
import org.wsmo.common.IRI;
import org.wsmo.common.Namespace;
import org.wsmo.factory.Factory;
import org.wsmo.factory.WsmoFactory;
import org.wsmo.service.Goal;
import org.wsmo.service.WebService;

public class SDCGraphMaintenanceTest extends TestCase {

	private WsmoFactory wsmoFactory; 
	
 	protected static Logger logger;
	private SDCResourceManager theResourceManager; 

	private SDCGraphCreator theCreator; 
	private SDCGraphEvolutionManager evolutionMgr; 

	private SDCGraph2DOT dotCreator; 

    protected void setUp() throws Exception {
    	super.setUp();
    	
    	wsmoFactory = Factory.createWsmoFactory(null);

		theResourceManager = new SDCResourceManager();
		theCreator = new SDCGraphCreator();
		evolutionMgr = new SDCGraphEvolutionManager();
		
		dotCreator = new SDCGraph2DOT(); 

		logger = Logger.getLogger(SDCGraphCreator.class);
		
    }
    
    private File createTestWS() {
    	
    	String testWSDirectory = "SDCcreationTestWS";

    	logger.info("trying to create Directory: " + testWSDirectory ); 
    	
    	File wsRepository = new File(testWSDirectory);	
		if (!wsRepository.exists()) {
	    	boolean success = wsRepository.mkdir();
	    	if (success) {
	    		logger.info("Directory: " + wsRepository.getAbsolutePath() + " created");
	    	}    
			
		}
		
    	
    	logger.info("store Web Services in " + wsRepository ); 
    	
    	String wsRepositoryFullPath = wsRepository.getAbsolutePath()+wsRepository.separator; 
    	
		WebService wsMuller = theResourceManager.loadWS("wsMuller.wsml");
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsMuller.wsml", wsMuller); 
		WebService wsRacer = theResourceManager.loadWS("wsRacer.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsRacer.wsml", wsRacer); 
		WebService wsRunner = theResourceManager.loadWS("wsRunner.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsRunner.wsml", wsRunner); 
		WebService wsWalker = theResourceManager.loadWS("wsWalker.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsWalker.wsml", wsWalker); 
		WebService wsWeasel = theResourceManager.loadWS("wsWeasel.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsWeasel.wsml", wsWeasel); 
		WebService wsOther = theResourceManager.loadWS("wsOther.wsml"); 
		theResourceManager.storeWebService(wsRepositoryFullPath, "wsOther.wsml", wsOther); 
		


	    for (int i = 2; i < 46 ; i++) {
	    	
			Namespace wsNamespace = wsmoFactory.createNamespace("testWS",
					wsmoFactory.createIRI("http://members.deri.org/~michaels/sdc/swsc-shipment/webservices/")); 
			
			WebService theOriginalWSother = theResourceManager.loadWS("wsOther.wsml");

	    	
	    	String wsID = "wsOther" + i + ".wsml"; 
	    	
			WebService anotherWS = wsmoFactory.createWebService(
					wsmoFactory.createIRI(wsNamespace, wsID));	
			Set<Namespace> theNS = theOriginalWSother.listNamespaces();		
			for (Namespace ns : theNS) {
				anotherWS.addNamespace(ns); 
			}
			Set<Ontology> theOntologies = theOriginalWSother.listOntologies(); 
			for (Ontology ont : theOntologies) {
				anotherWS.addOntology(ont); 
			}
			anotherWS.setCapability(theOriginalWSother.getCapability()); 
			
			theResourceManager.storeWebService(wsRepository.getAbsolutePath()+wsRepository.separator, wsID, anotherWS);
	    } 		
    	
	    return wsRepository; 
    		
    }
    
    private String getLocalGTname(Goal goalTemplate) {

    	IRI fullID = (IRI) goalTemplate.getIdentifier();
    	String GTlocalName = fullID.getLocalName();
    	
    	return GTlocalName; 

    }

    private String getLocalWSname(WebService webservice) {

    	IRI fullID = (IRI) webservice.getIdentifier();
    	String localName = fullID.getLocalName();
    	
    	return localName; 

    }

    private void testRunner(Goal goalTemplate, Ontology sdcGraph, String wsRepository, String theOutputDirectory, BufferedWriter bwData, File csvFile) {

		logger.info( "-----------insertion of goal template: " + getLocalGTname(goalTemplate) +"--------------" ); 
    	
        long startTime;
        long stopTime;
            
        startTime = System.currentTimeMillis();
        stopTime = startTime; 
            
        sdcGraph = theCreator.insertGoalTemplate(sdcGraph, goalTemplate, wsRepository); 	

        stopTime = System.currentTimeMillis();
        long duration = (stopTime - startTime); // in msec        
            
        logger.info("-----------insertion of " + getLocalGTname(goalTemplate) + " completed,  time: " + duration + " msec \n");
		     
        try {        	
            bwData.write("insertion of " + getLocalGTname(goalTemplate) + "," + duration + " \n"); 
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
            

    	
    }

      
    
    public void testSDCGraphCreator(){
    	
    	String outputDirectory = "outputs/"; 
    	
    	String resourceDirectory = "resourcesSWSCshipment/"; 

    	// creating the repository of available Web services 
    	File testWSrepositoryFile = createTestWS(); 
    	String testWSrepository = testWSrepositoryFile.getAbsolutePath()+testWSrepositoryFile.separator;    	
    	logger.info("used WS Repository : " + testWSrepository);
   	
	    // loading goal templates 
		Goal gtRoot = theResourceManager.loadGoalTemplate("gtRoot.wsml"); 
    	Goal gt2 = theResourceManager.loadGoalTemplate("gtUS2world.wsml");
    	Goal gt3 = theResourceManager.loadGoalTemplate("gtUS2AF.wsml");	
    	Goal gt4 = theResourceManager.loadGoalTemplate("gtUS2AS.wsml");
    	Goal gt5 = theResourceManager.loadGoalTemplate("gtUS2EU.wsml");
    	Goal gt6 = theResourceManager.loadGoalTemplate("gtUS2EUlight.wsml");
    	Goal gt7 = theResourceManager.loadGoalTemplate("gtUS2NA.wsml");
    	Goal gt8 = theResourceManager.loadGoalTemplate("gtUS2SA.wsml");
    	Goal gt9 = theResourceManager.loadGoalTemplate("gtUS2OC.wsml");
    	Goal gt10 = theResourceManager.loadGoalTemplate("gtNA2NAlight.wsml");
    	
     	
   		
		
    	
      	logger.info( "----------- SDC GRAPH MAINTENANCE TESTS --------------" );
      	
		// set-up CSV file for capturing time 
        File csvFile3 = new File(outputDirectory, "SDCGraphMaintenanceTests.csv");        
        BufferedWriter bwData3;
        
        try {    		
    		bwData3 = new BufferedWriter(new FileWriter(csvFile3));
    		bwData3.write("Times for SDC Graph Maintenance Operations \n \n " );   
    		bwData3.write("ACTION, TIME (in msec) \n " );   

    		
            long startTime;
            long stopTime;                
            
        	logger.info( "----------- deletion of a goal template  --------------" );

        	Ontology maintenanceSCDGraph1 = theResourceManager.loadSDCGraph( resourceDirectory, "sdcGraphOntology-complete.wsml");         	

        	startTime = System.currentTimeMillis();
            stopTime = startTime;

            maintenanceSCDGraph1 = evolutionMgr.removeGoalTemplate(gt5, maintenanceSCDGraph1); 

            stopTime = System.currentTimeMillis();
            long duration1 = (stopTime - startTime); // in msec        
            
            bwData3.write("deletion of " + getLocalGTname(gt5) + "," + duration1 + " \n");

            logger.info("-----------deletion of " + getLocalGTname(gt5) + " completed,  time: " + duration1 + " msec \n");

    		// store the created SDC graph ontology 
        	String maintenanceSDCGraphFileName1 = "sdcGraphOntology-afterGTremoval.wsml";
    		theResourceManager.storeSDCGraph(outputDirectory, maintenanceSDCGraphFileName1, maintenanceSCDGraph1); 
    		// also DOT file creation 
    		dotCreator.createDOT(outputDirectory, "sdcGraphOntology-afterGTremoval.wsml", outputDirectory, "sdcGraphOntology-afterGTremoval"); 

    		

        	logger.info( "----------- deletion of a goal template with direct iGT  --------------" );

        	Ontology maintenanceSCDGraph2 = theResourceManager.loadSDCGraph(outputDirectory, "sdcGraphOntology-afterGTremoval.wsml");         	

        	startTime = System.currentTimeMillis();
            stopTime = startTime;

            maintenanceSCDGraph2 = evolutionMgr.removeGoalTemplate(gt10, maintenanceSCDGraph2); 

            stopTime = System.currentTimeMillis();
            long duration2 = (stopTime - startTime); // in msec        
            
            bwData3.write("deletion of " + getLocalGTname(gt10) + "," + duration2 + " \n");

            logger.info("-----------deletion of " + getLocalGTname(gt10) + " completed,  time: " + duration2 + " msec \n");

    		// store the created SDC graph ontology 
        	String maintenanceSDCGraphFileName2 = "sdcGraphOntology-afterGTremoval2.wsml";
    		theResourceManager.storeSDCGraph(outputDirectory, maintenanceSDCGraphFileName2, maintenanceSCDGraph2); 
    		// also DOT file creation 
    		dotCreator.createDOT(outputDirectory, "sdcGraphOntology-afterGTremoval2.wsml", outputDirectory, "sdcGraphOntology-afterGTremoval2"); 
    		

            
        	logger.info( "----------- deletion of a Web Service  --------------" );
        	
        	Ontology maintenanceSCDGraph3 = theResourceManager.loadSDCGraph(outputDirectory, "sdcGraphOntology-afterGTremoval2.wsml"); 
        	WebService theRemovedWS = theResourceManager.loadWS("wsMuller.wsml");
        	
        	startTime = System.currentTimeMillis();
            stopTime = startTime;

        	maintenanceSCDGraph3 = evolutionMgr.removeWebService(theRemovedWS, maintenanceSCDGraph3); 

            stopTime = System.currentTimeMillis();
            long duration3 = (stopTime - startTime); // in msec        

            bwData3.write("deletion of " + getLocalWSname(theRemovedWS) + "," + duration3 + " \n");

            logger.info("-----------deletion of " + getLocalWSname(theRemovedWS) + " completed,  time: " + duration3 + " msec \n");

    		// store the created SDC graph ontology 
        	String SDCGraphFileName4 = "sdcGraphOntology-afterWSremoval.wsml";
    		theResourceManager.storeSDCGraph(outputDirectory, SDCGraphFileName4, maintenanceSCDGraph3); 
    		// also DOT file creation 
    		dotCreator.createDOT(outputDirectory, "sdcGraphOntology-afterWSremoval.wsml", outputDirectory, "sdcGraphOntology-afterWSremoval"); 
		

        	logger.info( "----------- insertion of a Web Service  --------------" );

        	
        	Ontology maintenanceSCDGraph4 = theResourceManager.loadSDCGraph(outputDirectory, "sdcGraphOntology-afterWSremoval.wsml"); 
        	WebService theInsertedWS = theResourceManager.loadWS("wsMuller.wsml");
        	
        	startTime = System.currentTimeMillis();
            stopTime = startTime;

        	maintenanceSCDGraph4 = evolutionMgr.insertWebService(theInsertedWS, maintenanceSCDGraph4); 
        	
            stopTime = System.currentTimeMillis();
            long duration4 = (stopTime - startTime); // in msec        
            
            bwData3.write("(re)insertion of " + getLocalWSname(theInsertedWS) + "," + duration4 + " \n");

            logger.info("-----------(re)insertion of " + getLocalWSname(theInsertedWS) + " completed,  time: " + duration4 + " msec \n");


    		// store the created SDC graph ontology 
        	String SDCGraphFileName5 = "sdcGraphOntology-afterWSinsertion.wsml";
    		theResourceManager.storeSDCGraph(outputDirectory, SDCGraphFileName5, maintenanceSCDGraph4); 
    		// also DOT file creation 
    		dotCreator.createDOT(outputDirectory, "sdcGraphOntology-afterWSinsertion.wsml", outputDirectory, "sdcGraphOntology-afterWSinsertion"); 

    		
    		// close the CSV file 
    		bwData3.flush();
            bwData3.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
 
    }
}
