/**
 * this creates the 10 goal instances for the SDC performance comparison
 * - ensures that the defined GIs are valid / processable 
 * - stores GIs in a separate WSML file
 * 
 * @author Michael Stollberg
 *
 * @version $Revision: 1.1 $ $Date: 2007/10/10 19:54:01 $
 * 
 */
package org.deri.wsmx.discovery.caching;

import java.util.HashMap;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.deri.wsmx.discovery.caching.matchmaking.Matchmaking4SWSC;
import org.omwg.ontology.Instance;
import org.omwg.ontology.Ontology;
import org.wsmo.factory.Factory;
import org.wsmo.factory.WsmoFactory;
import org.wsmo.service.Goal;
import org.wsmo.service.WebService;

public class SWSCshipmentGoalInstancesCreation extends TestCase {
	
	private GoalInstanceManager giManager; 
	private SDCResourceManager resourceMgr; 
	
 	protected static Logger logger;


    protected void setUp() throws Exception {
    	super.setUp();

		logger = Logger.getLogger(SWSCshipmentGoalInstancesCreation.class);

    	giManager = new GoalInstanceManager(); 
		resourceMgr = new SDCResourceManager(); 

	}
    

    public void testGIManager() {
            logger.info("This creates the 10 Goal Instances for the SDC Performance Evaluation Tests " +
            		"also, all the usable Web services for each Goal Instances are determined " +
            		"for the extended SWSC shipment scenario");
            
            
            Ontology theGIontology = resourceMgr.loadSDCGraphSchema();

            logger.info(" --------------------------------" );

            String[] inputs4GI1 = {"california","tunis","1.0"};
            Goal gt4GI1 = resourceMgr.loadGoalTemplate("gtUS2AF.wsml");
            Instance GI1 = giManager.createGoalInstance(inputs4GI1, gt4GI1, theGIontology);
            boolean isValidGI1 = giManager.GIValidation(GI1, gt4GI1, theGIontology);
            logger.info("  created: " + GI1.getIdentifier() + " \n  is valid: " + isValidGI1 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI1, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI2 = {"california","luxembourgCity","1.5"};
            Goal gt4GI2 = resourceMgr.loadGoalTemplate("gtRoot.wsml"); 
            Instance GI2 = giManager.createGoalInstance(inputs4GI2, gt4GI2, theGIontology);
            boolean isValidGI2 = giManager.GIValidation(GI2, gt4GI2, theGIontology);
            logger.info("  created: " + GI2.getIdentifier() + " \n  is valid: " + isValidGI2 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI2, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI3 = {"california","tunis","50.5"};
            Goal gt4GI3 = resourceMgr.loadGoalTemplate("gtUS2AF.wsml");
            Instance GI3 = giManager.createGoalInstance(inputs4GI3, gt4GI3, theGIontology);
            boolean isValidGI3 = giManager.GIValidation(GI3, gt4GI3, theGIontology);
            logger.info("  created: " + GI3.getIdentifier() + " \n  is valid: " + isValidGI3 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI3, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI4 = {"california","bristol","4.3"};
            Goal gt4GI4 = resourceMgr.loadGoalTemplate("gtUS2EU.wsml");
            Instance GI4 = giManager.createGoalInstance(inputs4GI4, gt4GI4, theGIontology);
            boolean isValidGI4 = giManager.GIValidation(GI4, gt4GI4, theGIontology);
            logger.info("  created: " + GI4.getIdentifier() + " \n  is valid: " + isValidGI4 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI4, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI5 = {"california","newYorkCity","5.5"};
            Goal gt4GI5 = resourceMgr.loadGoalTemplate("gtUS2NA.wsml");
            Instance GI5 = giManager.createGoalInstance(inputs4GI5, gt4GI5, theGIontology);
            boolean isValidGI5 = giManager.GIValidation(GI5, gt4GI5, theGIontology);
            logger.info("  created: " + GI5.getIdentifier() + " \n  is valid: " + isValidGI5 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI5, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI6 = {"california","berlin","6.75"};
            Goal gt4GI6 = resourceMgr.loadGoalTemplate("gtUS2world.wsml");
            Instance GI6 = giManager.createGoalInstance(inputs4GI6, gt4GI6, theGIontology);
            boolean isValidGI6 = giManager.GIValidation(GI6, gt4GI6, theGIontology);
            logger.info("  created: " + GI6.getIdentifier() + " \n  is valid: " + isValidGI6 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI6, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI7 = {"california","sydney","17.3"};
            Goal gt4GI7 = resourceMgr.loadGoalTemplate("gtUS2world.wsml");
            Instance GI7 = giManager.createGoalInstance(inputs4GI7, gt4GI7, theGIontology);
            boolean isValidGI7 = giManager.GIValidation(GI7, gt4GI7, theGIontology);
            logger.info("  created: " + GI7.getIdentifier() + " \n  is valid: " + isValidGI7 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI7, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI8 = {"california","quito","7.58"};
            Goal gt4GI8 = resourceMgr.loadGoalTemplate("gtUS2SA.wsml");
            Instance GI8 = giManager.createGoalInstance(inputs4GI8, gt4GI8, theGIontology);
            boolean isValidGI8 = giManager.GIValidation(GI8, gt4GI8, theGIontology);
            logger.info("  created: " + GI8.getIdentifier() + " \n  is valid: " + isValidGI8 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI8, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI9 = {"california","beijing","7.8"};
            Goal gt4GI9 = resourceMgr.loadGoalTemplate("gtUS2AS.wsml");
            Instance GI9 = giManager.createGoalInstance(inputs4GI9, gt4GI9, theGIontology);
            boolean isValidGI9 = giManager.GIValidation(GI9, gt4GI9, theGIontology);
            logger.info("  created: " + GI9.getIdentifier() + " \n  is valid: " + isValidGI9 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI9, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

            logger.info(" --------------------------------" );

            String[] inputs4GI10 = {"california","amsterdam","9.99"};
            Goal gt4GI10 = resourceMgr.loadGoalTemplate("gtUS2EUlight.wsml");
            Instance GI10 = giManager.createGoalInstance(inputs4GI10, gt4GI10, theGIontology);
            boolean isValidGI10 = giManager.GIValidation(GI10, gt4GI10, theGIontology);
            logger.info("  created: " + GI10.getIdentifier() + " \n  is valid: " + isValidGI10 );

            for (WebService aWS : giManager.findAllWS4GIextendedSWSC(GI10, theGIontology)) {
                logger.info("  usable WS: " + aWS.getIdentifier() );
            }

	    	// storing ... 
	    	
	    	String SDCGraphDirectory = "outputs/";
	    	String filename = "goalInstances4SWSC.wsml"; 
	    	
	    	resourceMgr.storeSDCGraph(SDCGraphDirectory, filename , theGIontology); 

	    	

	    	
	    	
	    }


}
